@@ -71,6 +71,9 @@ from kipy.proto.board.board_pb2 import (    # noqa
 from kipy.proto.board.board_types_pb2 import ( #noqa
     BoardLayer
 )
+from kipy.proto.board.board_commands_pb2 import ( #noqa
+    BoardOriginType
+)
 
 class BoardLayerGraphicsDefaults(Wrapper):
     """The default properties for graphic items added on a given class of board layer"""
@@ -444,10 +447,13 @@ class Board:
         command = editor_commands_pb2.SaveSelectionToString()
         return self._kicad.send(command, editor_commands_pb2.SavedSelectionResponse).contents
 
-    def update_items(self, items: Union[BoardItem, Sequence[BoardItem]]):
+    def update_items(self, items: Union[BoardItem, Sequence[BoardItem]]) -> List[BoardItem]:
         """Updates the properties of one or more items on the board.  The items must already exist
         on the board, and are matched by internal UUID.  All other properties of the items are
-        updated from those passed in this call."""
+        updated from those passed in this call.
+
+        Returns the updated items, which may be different from the input items if any updates
+        failed to apply (for example, if any properties were out of range and were clamped)"""
         command = UpdateItems()
         command.header.document.CopyFrom(self._doc)
 
@@ -457,12 +463,16 @@ class Board:
             command.items.extend([pack_any(i.proto) for i in items])
 
         if len(command.items) == 0:
-            return
+            return []
 
-        return [
-            unwrap(result.item)
-            for result in self._kicad.send(command, UpdateItemsResponse).updated_items
-        ]
+        return self._to_concrete_items(
+            [
+                unwrap(result.item)
+                for result in self._kicad.send(
+                    command, UpdateItemsResponse
+                ).updated_items
+            ]
+        )
 
     def remove_items(self, items: Union[BoardItem, Sequence[BoardItem]]):
         """Deletes one or more items from the board"""
@@ -479,6 +489,23 @@ class Board:
 
         self._kicad.send(command, DeleteItemsResponse)
 
+    def remove_items_by_id(self, items: Union[KIID, Sequence[KIID]]):
+        """Deletes one or more items from the board using their unique IDs
+
+        .. versionadded:: 0.4.0"""
+        command = DeleteItems()
+        command.header.document.CopyFrom(self._doc)
+
+        if isinstance(items, KIID):
+            command.item_ids.append(items)
+        else:
+            command.item_ids.extend(items)
+
+        if len(command.item_ids) == 0:
+            return
+
+        self._kicad.send(command, DeleteItemsResponse)
+
     def get_nets(
         self, netclass_filter: Optional[Union[str, Sequence[str]]] = None
     ) -> Sequence[Net]:
@@ -608,6 +635,26 @@ class Board:
         cmd.document.CopyFrom(self._doc)
         return TitleBlockInfo(self._kicad.send(cmd, base_types_pb2.TitleBlockInfo))
 
+    def get_origin(self, origin_type: board_commands_pb2.BoardOriginType.ValueType) -> Vector2:
+        """Retrieves the specified (grid or drill/place) board origin
+
+        .. versionadded:: 0.3.0"""
+        cmd = board_commands_pb2.GetBoardOrigin()
+        cmd.board.CopyFrom(self._doc)
+        cmd.type = origin_type
+        return Vector2(self._kicad.send(cmd, base_types_pb2.Vector2))
+
+    def set_origin(self, origin_type: board_commands_pb2.BoardOriginType.ValueType,
+                   origin: Vector2):
+        """Sets the specified (grid or drill/place) board origin
+
+        .. versionadded:: 0.3.0"""
+        cmd = board_commands_pb2.SetBoardOrigin()
+        cmd.board.CopyFrom(self._doc)
+        cmd.type = origin_type
+        cmd.origin.CopyFrom(origin.proto)
+        self._kicad.send(cmd, Empty)
+
     @overload
     def expand_text_variables(self, text: str) -> str:
         ...
@@ -712,6 +759,55 @@ class Board:
             if p is not None
         ]
 
+    def check_padstack_presence_on_layers(
+        self,
+        items: Union[BoardItem, Iterable[BoardItem]],
+        layers: Union[board_types_pb2.BoardLayer.ValueType, Iterable[board_types_pb2.BoardLayer.ValueType]]
+    ) -> Dict[BoardItem, Dict[board_types_pb2.BoardLayer.ValueType, bool]]:
+        """Checks if the given items with padstacks (pads or vias) have content on the given layers.
+
+        :param items: The items to check (one or more pads or vias).
+        :param layers: The layer or layers to check for padstack presence.
+        :return: A dictionary mapping each item to a dictionary of layers and their presence on
+                 the given layer.
+
+        .. versionadded:: 0.4.0 with KiCad 9.0.3
+        """
+        cmd = board_commands_pb2.CheckPadstackPresenceOnLayers()
+        cmd.board.CopyFrom(self._doc)
+
+        items_map = {}
+
+        if isinstance(items, BoardItem):
+            cmd.items.append(items.id)
+            items_map[items.id.value] = items
+        else:
+            cmd.items.extend([item.id for item in items])
+            items_map.update({item.id.value: item for item in items})
+
+        if isinstance(layers, int):
+            cmd.layers.append(layers)
+        else:
+            cmd.layers.extend(layers)
+
+        response = self._kicad.send(cmd, board_commands_pb2.PadstackPresenceResponse)
+
+        result = {}
+        for entry in response.entries:
+            if entry.item.value not in items_map:
+                continue
+
+            item = items_map[entry.item.value]
+            layer = entry.layer
+            presence = entry.presence is board_commands_pb2.PadstackPresence.PSP_PRESENT
+
+            if item not in result:
+                result[item] = {}
+
+            result[item][layer] = presence
+
+        return result
+
     def interactive_move(self, items: Union[KIID, Iterable[KIID]]):
         """Initiates an interactive move operation on one or more items on the board.  The user
         will be able to move the items interactively in the KiCad editor.  This is a blocking
